/***********************************************************************
 **  scalespace.h
 ***********************************************************************/

#if !defined(SCALESPACE_H)
#define SCALESPACE_H 1

#include "image.h"

namespace stdi {

  // ZC: zero crossing, EF: elliptic feature, LE: localized edge
  enum FeatureType {
    
    BrightZC = 0x0001,
    DarkZC   = 0x0002,
    BrightEF = 0x0004,
    DarkEF   = 0x0008,
    BrightLE = 0x0010,
    DarkLE   = 0x0020
  };

  namespace scalespace {

    
    // "smooth" is a special case of "gaussian" (order_x=order_y=order_z=0)
    // However for better boundary accuracy, please use smooth 
    FloatImage gaussian(const Image &img, float scale, 
			int order_x=0, int order_y=0, int order_z=0);
    FloatImage gaussian(const Image &img, float scale_x, float scale_y, float scale_z, 
			int order_x=0, int order_y = 0, int order_z=0);
    FloatImage smooth(const Image &img, float scale, const float ratio = 1.0);
    FloatImage smooth(const Image &img, float scale_x, float scale_y, float scale_z);
    
    // Partial derivatives
    FloatImage dX(const Image &img, float scale);
    FloatImage dY(const Image &img, float scale);
    FloatImage dZ(const Image &img, float scale);
    FloatImage dXX(const Image &img, float scale);
    FloatImage dYY(const Image &img, float scale);
    FloatImage dZZ(const Image &img, float scale);
    FloatImage dXY(const Image &img, float scale);
    FloatImage dXY(const Image &img, float x_scale, float y_scale);
    FloatImage dXZ(const Image &img, float scale);
    FloatImage dXZ(const Image &img, float x_scale, float z_scale);
    FloatImage dYZ(const Image &img, float scale);
    FloatImage dYZ(const Image &img, float y_scale, float z_scale);

    FloatImage dXYZ(const Image &img, float x_scale, float y_scale, float z_scale);
    FloatImage dXXX(const Image &img, float x_scale);
    FloatImage dYYY(const Image &img, float y_scale);
    FloatImage dZZZ(const Image &img, float z_scale);
    FloatImage dXXY(const Image &img, float x_scale, float y_scale);
    FloatImage dXXZ(const Image &img, float x_scale, float z_scale);
    FloatImage dXYY(const Image &img, float x_scale, float y_scale);
    FloatImage dYYZ(const Image &img, float y_scale, float z_scale);
    FloatImage dXZZ(const Image &img, float x_scale, float z_scale);
    FloatImage dYZZ(const Image &img, float y_scale, float z_scale);

    FloatImage gradient(const Image &img, float scale, const float ratio = 1.0);
    FloatImage laplacian(const Image &img, float scale);
    

    FloatImage angleInRadian(const Image &Ix, const Image &Iy);
    ShortImage angle(const Image &Ix, const Image &Iy);
    ShortImage gradient(const Image &Ix, const Image &Iy);
    ShortImage gradient(const Image &Ix, const Image &Iy, const Image& Iz);
    ByteImage direction(const Image &Ix, const Image &Iy);
    

    // Local features
    ByteImage features(const Image &img, float scale,  FeatureType ft, const float ratio = 1.0);
    
    // The deg-th derivative of Gaussian kernel on [-w, w]
    float *gaussianKernel(float scale, int &w, int deg);
    vector<float> dog(int m, float sigma);
    float polyval(const vector<float> &p, float x);
    
    // Nonlinear and structural diffusion
    FloatImage harmonicCut(const Image &I0, float scale, 
			   bool remove_bright_regions = true,
			   int niter = 10);

    ByteImage hcut(const Image &I0, float scale, float zc_scale=1.5);
    void laplaceEquation(FloatImage &I, const ByteImage &E);
    
  } // End of namespace scalespace

} // End of namespace stdi
 
#endif // SCALE_SPACE_3D_H
